/**
* Header file of ML CFP2 API library.
*/
#pragma once
#ifndef ML_CFP2_H__
#define ML_CFP2_H__

#ifndef APICALL
#ifdef __cplusplus
#   ifdef WIN32
#       ifndef ML_IMPORTS_API
#           define APICALL extern "C" __declspec(dllexport)
#       else
#           define APICALL extern "C" __declspec(dllimport)
#       endif
#   else
#       define APICALL extern "C" 
#   endif // WIN32
#else
#   define  APICALL
#endif
#endif
#ifdef __cplusplus
#define  CFP2STACKMODE   __stdcall
#else
#define  CFP2STACKMODE
#endif

#define MAXDEVNUM   32      // Maximum number of attached devices

enum { MLCFP2WRAPPERVERSION = 1 };  // Incremental module version

struct mlcfp2;  // ML CFP2 API wrapper structure.

typedef unsigned short ushort;     // Unsigned 16-bit integer
typedef unsigned char byte;
typedef unsigned long UInt32;

/// <summary>Enumerate attached devices.</summary>
/// <param name="devices">Output array of detected instance numbers. Array's expected length is `MAXDEVNUM`.</param>
/// <param name="serialNum">Output array of detected serial numbers.Array's expected length is `MAXDEVNUM`.</param>
/// <returns>Number of detected devices.</returns>
APICALL int CFP2STACKMODE mlcfp2_enumerateDevices(ushort devices[], byte serialNum[]);

/// <summary>Create a CFP2 API wrapper instance.</summary>
/// <param name="instNum">USB instance number.</param>
/// <returns>Pointer to created API wrapper.</returns>
APICALL mlcfp2* CFP2STACKMODE mlcfp2_createInstance(ushort instNum);

/// <summary>Destroy a CFP2 API wrapper instance.</summary>
/// <param name="inst">Wrapper instance.</param>
APICALL void CFP2STACKMODE mlcfp2_destroyInstance(mlcfp2* inst);

/// <summary>Connect to an instance.</summary>
/// <param name="inst">Wrapper instance.</param>
APICALL bool CFP2STACKMODE mlcfp2_connect(mlcfp2* inst);

/// <summary>Disconnect from an instance.</summary>
/// <param name="inst">Wrapper instance.</param>
APICALL bool CFP2STACKMODE mlcfp2_disconnect(mlcfp2* inst);

/// <summary>Read EEPROM (Needs a FW update).</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="address">EEPROM Address.</param>
/// <param name="Value">EEPROM Value.</param>
APICALL int CFP2STACKMODE mlcfp2_eepromRead(mlcfp2* inst, UInt32 address, byte *Value);

/// <summary>Write EEPROM (Needs a FW update).</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="address">EEPROM Address.</param>
/// <param name="Value">EEPROM Value.</param>
APICALL int CFP2STACKMODE mlcfp2_eepromWrite(mlcfp2* inst, UInt32 address, byte Value);

/// <summary>Current Monitor.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="data">Current Value in mA.</param>>
APICALL bool CFP2STACKMODE mlcfp2_p3v3CurrentMonitor(mlcfp2* inst, ushort* data);

/// <summary>MODABS pin status.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="status">true = Module Absent, false = Module Present.</param>>
APICALL bool CFP2STACKMODE mlcfp2_modABS(mlcfp2* inst, bool* status);

/// <summary>RXLOS pin status.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="status">true = pin high, false = pin low.</param>>
APICALL bool CFP2STACKMODE mlcfp2_rxLOS(mlcfp2* inst, bool* status);

/// <summary>GLBALRM pin status.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="status">true = pin high, false = pin low.</param>>
APICALL bool CFP2STACKMODE mlcfp2_glbALRM(mlcfp2* inst, bool* status);

/// <summary>PRGALRM1 pin status.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="status">true = pin high, false = pin low.</param>>
APICALL bool CFP2STACKMODE mlcfp2_prgALRM1(mlcfp2* inst, bool* status);

/// <summary>PRGALRM2 pin status.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="status">true = pin asserted, false = pin deasserted.</param>>
APICALL bool CFP2STACKMODE mlcfp2_prgALRM2(mlcfp2* inst, bool* status);

/// <summary>PRGALRM3 pin status.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="status">true = pin asserted, false = pin deasserted.</param>>
APICALL bool CFP2STACKMODE mlcfp2_prgALRM3(mlcfp2* inst, bool* status);

/// <summary>TXDIS pin assertion.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="isAsserted">true = pin asserted, false = pin deasserted.</param>>
APICALL bool CFP2STACKMODE mlcfp2_txDIS(mlcfp2* inst, bool isAsserted);

/// <summary>MODLOPWR pin assertion.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="isAsserted">true = pin asserted, false = pin deasserted.</param>>
APICALL bool CFP2STACKMODE mlcfp2_modLOPWR(mlcfp2* inst, bool isAsserted);

/// <summary>MODRST pin assertion.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="isAsserted">true = pin asserted, false = pin deasserted.</param>>
APICALL bool CFP2STACKMODE mlcfp2_modRST(mlcfp2* inst, bool isAsserted);

/// <summary>PRGCNTL1 pin assertion.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="isAsserted">true = pin asserted, false = pin deasserted.</param>>
APICALL bool CFP2STACKMODE mlcfp2_prgCNTL1(mlcfp2* inst, bool isAsserted);

/// <summary>PRGCNTL2 pin assertion.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="isAsserted">true = pin asserted, false = pin deasserted.</param>>
APICALL bool CFP2STACKMODE mlcfp2_prgCNTL2(mlcfp2* inst, bool isAsserted);

/// <summary>PRGCNTL3 pin assertion.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="isAsserted">true = pin asserted, false = pin deasserted.</param>>
APICALL bool CFP2STACKMODE mlcfp2_prgCNTL3(mlcfp2* inst, bool isAsserted);

/// <summary>Sets MDIO Physical Port Address.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="Value">Port Value.</param>>
APICALL bool CFP2STACKMODE mlcfp2_setPRTADR(mlcfp2* inst, byte Value);

/// <summary>Gets Device SN  (Needs a FW update).</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="serialNum">SN returned value.</param>>
APICALL bool CFP2STACKMODE mlcfp2_getSN(mlcfp2* inst, byte serialNum[]);

/// <summary>Sets a Device SN  (Needs a FW update).</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="serialNum">SN Value.</param>>
APICALL bool CFP2STACKMODE mlcfp2_setSN(mlcfp2* inst, byte serialNum[], int SNsize);

/// <summary>Read MDIO.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="DeviceAddress">Device Address.</param>>
/// <param name="RegisterAddress">Register Address to read from.</param>>
/// <param name="data">returned register value.</param>>
APICALL bool CFP2STACKMODE mlcfp2_readMDIO(mlcfp2* inst, byte DeviceAddress, ushort RegisterAddress, ushort* data);

/// <summary>Write MDIO.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="DeviceAddress">Device Address.</param>>
/// <param name="RegisterAddress">Register address to write to.</param>>
/// <param name="data">Value to be written.</param>>
APICALL bool CFP2STACKMODE mlcfp2_writeMDIO(mlcfp2* inst, byte DeviceAddress, ushort RegisterAddress, ushort data);
#endif // !ML_CFP2_H__

